from typing import ClassVar as _ClassVar
from typing import Iterable as _Iterable
from typing import Mapping as _Mapping
from typing import Optional as _Optional
from typing import Union as _Union

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import wrappers_pb2 as _wrappers_pb2
from google.protobuf.internal import containers as _containers
from omni.pro.protos.common import base_pb2 as _base_pb2
from omni.pro.protos.v1.rules import location_pb2 as _location_pb2
from omni.pro.protos.v1.rules import warehouse_pb2 as _warehouse_pb2

DESCRIPTOR: _descriptor.FileDescriptor

class WarehouseHierarchy(_message.Message):
    __slots__ = [
        "name",
        "code",
        "warehouse",
        "location",
        "quantity_security",
        "sequence",
        "sequence_order",
        "gift_code",
    ]
    NAME_FIELD_NUMBER: _ClassVar[int]
    CODE_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_FIELD_NUMBER: _ClassVar[int]
    LOCATION_FIELD_NUMBER: _ClassVar[int]
    QUANTITY_SECURITY_FIELD_NUMBER: _ClassVar[int]
    SEQUENCE_FIELD_NUMBER: _ClassVar[int]
    SEQUENCE_ORDER_FIELD_NUMBER: _ClassVar[int]
    GIFT_CODE_FIELD_NUMBER: _ClassVar[int]
    name: str
    code: str
    warehouse: _warehouse_pb2.Warehouse
    location: _location_pb2.Location
    quantity_security: float
    sequence: int
    sequence_order: _wrappers_pb2.BoolValue
    gift_code: str
    def __init__(
        self,
        name: _Optional[str] = ...,
        code: _Optional[str] = ...,
        warehouse: _Optional[_Union[_warehouse_pb2.Warehouse, _Mapping]] = ...,
        location: _Optional[_Union[_location_pb2.Location, _Mapping]] = ...,
        quantity_security: _Optional[float] = ...,
        sequence: _Optional[int] = ...,
        sequence_order: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        gift_code: _Optional[str] = ...,
    ) -> None: ...

class WarehouseHierarchyCreate(_message.Message):
    __slots__ = [
        "name",
        "code",
        "warehouse_id",
        "location_id",
        "quantity_security",
        "sequence",
        "sequence_order",
        "gift_code",
    ]
    NAME_FIELD_NUMBER: _ClassVar[int]
    CODE_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_ID_FIELD_NUMBER: _ClassVar[int]
    LOCATION_ID_FIELD_NUMBER: _ClassVar[int]
    QUANTITY_SECURITY_FIELD_NUMBER: _ClassVar[int]
    SEQUENCE_FIELD_NUMBER: _ClassVar[int]
    SEQUENCE_ORDER_FIELD_NUMBER: _ClassVar[int]
    GIFT_CODE_FIELD_NUMBER: _ClassVar[int]
    name: str
    code: str
    warehouse_id: int
    location_id: int
    quantity_security: float
    sequence: int
    sequence_order: _wrappers_pb2.BoolValue
    gift_code: str
    def __init__(
        self,
        name: _Optional[str] = ...,
        code: _Optional[str] = ...,
        warehouse_id: _Optional[int] = ...,
        location_id: _Optional[int] = ...,
        quantity_security: _Optional[float] = ...,
        sequence: _Optional[int] = ...,
        sequence_order: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        gift_code: _Optional[str] = ...,
    ) -> None: ...

class DeliveryWarehouse(_message.Message):
    __slots__ = [
        "id",
        "name",
        "hierarchy_warehouse_sort_by",
        "transfer_warehouses",
        "active",
        "external_id",
        "object_audit",
    ]
    ID_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    HIERARCHY_WAREHOUSE_SORT_BY_FIELD_NUMBER: _ClassVar[int]
    TRANSFER_WAREHOUSES_FIELD_NUMBER: _ClassVar[int]
    ACTIVE_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    OBJECT_AUDIT_FIELD_NUMBER: _ClassVar[int]
    id: str
    name: str
    hierarchy_warehouse_sort_by: str
    transfer_warehouses: _containers.RepeatedCompositeFieldContainer[WarehouseHierarchy]
    active: _wrappers_pb2.BoolValue
    external_id: str
    object_audit: _base_pb2.ObjectAudit
    def __init__(
        self,
        id: _Optional[str] = ...,
        name: _Optional[str] = ...,
        hierarchy_warehouse_sort_by: _Optional[str] = ...,
        transfer_warehouses: _Optional[_Iterable[_Union[WarehouseHierarchy, _Mapping]]] = ...,
        active: _Optional[_Union[_wrappers_pb2.BoolValue, _Mapping]] = ...,
        external_id: _Optional[str] = ...,
        object_audit: _Optional[_Union[_base_pb2.ObjectAudit, _Mapping]] = ...,
    ) -> None: ...

class DeliveryWarehouseCreateRequest(_message.Message):
    __slots__ = ["name", "hierarchy_warehouse_sort_by", "transfer_warehouses", "external_id", "context"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    HIERARCHY_WAREHOUSE_SORT_BY_FIELD_NUMBER: _ClassVar[int]
    TRANSFER_WAREHOUSES_FIELD_NUMBER: _ClassVar[int]
    EXTERNAL_ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    name: str
    hierarchy_warehouse_sort_by: str
    transfer_warehouses: _containers.RepeatedCompositeFieldContainer[WarehouseHierarchyCreate]
    external_id: str
    context: _base_pb2.Context
    def __init__(
        self,
        name: _Optional[str] = ...,
        hierarchy_warehouse_sort_by: _Optional[str] = ...,
        transfer_warehouses: _Optional[_Iterable[_Union[WarehouseHierarchyCreate, _Mapping]]] = ...,
        external_id: _Optional[str] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class DeliveryWarehouseCreateResponse(_message.Message):
    __slots__ = ["delivery_warehouse", "response_standard"]
    DELIVERY_WAREHOUSE_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    delivery_warehouse: DeliveryWarehouse
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        delivery_warehouse: _Optional[_Union[DeliveryWarehouse, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class DeliveryWarehouseReadRequest(_message.Message):
    __slots__ = ["group_by", "sort_by", "fields", "filter", "paginated", "id", "context"]
    GROUP_BY_FIELD_NUMBER: _ClassVar[int]
    SORT_BY_FIELD_NUMBER: _ClassVar[int]
    FIELDS_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    PAGINATED_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    group_by: _containers.RepeatedCompositeFieldContainer[_base_pb2.GroupBy]
    sort_by: _base_pb2.SortBy
    fields: _base_pb2.Fields
    filter: _base_pb2.Filter
    paginated: _base_pb2.Paginated
    id: str
    context: _base_pb2.Context
    def __init__(
        self,
        group_by: _Optional[_Iterable[_Union[_base_pb2.GroupBy, _Mapping]]] = ...,
        sort_by: _Optional[_Union[_base_pb2.SortBy, _Mapping]] = ...,
        fields: _Optional[_Union[_base_pb2.Fields, _Mapping]] = ...,
        filter: _Optional[_Union[_base_pb2.Filter, _Mapping]] = ...,
        paginated: _Optional[_Union[_base_pb2.Paginated, _Mapping]] = ...,
        id: _Optional[str] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class DeliveryWarehouseReadResponse(_message.Message):
    __slots__ = ["delivery_warehouses", "meta_data", "response_standard"]
    DELIVERY_WAREHOUSES_FIELD_NUMBER: _ClassVar[int]
    META_DATA_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    delivery_warehouses: _containers.RepeatedCompositeFieldContainer[DeliveryWarehouse]
    meta_data: _base_pb2.MetaData
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        delivery_warehouses: _Optional[_Iterable[_Union[DeliveryWarehouse, _Mapping]]] = ...,
        meta_data: _Optional[_Union[_base_pb2.MetaData, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class DeliveryWarehouseUpdateRequest(_message.Message):
    __slots__ = ["delivery_warehouse", "context"]
    DELIVERY_WAREHOUSE_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    delivery_warehouse: DeliveryWarehouse
    context: _base_pb2.Context
    def __init__(
        self,
        delivery_warehouse: _Optional[_Union[DeliveryWarehouse, _Mapping]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class DeliveryWarehouseUpdateResponse(_message.Message):
    __slots__ = ["delivery_warehouse", "response_standard"]
    DELIVERY_WAREHOUSE_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    delivery_warehouse: DeliveryWarehouse
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        delivery_warehouse: _Optional[_Union[DeliveryWarehouse, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class DeliveryWarehouseDeleteRequest(_message.Message):
    __slots__ = ["id", "context"]
    ID_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    id: str
    context: _base_pb2.Context
    def __init__(
        self, id: _Optional[str] = ..., context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...
    ) -> None: ...

class DeliveryWarehouseDeleteResponse(_message.Message):
    __slots__ = ["response_standard"]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    response_standard: _base_pb2.ResponseStandard
    def __init__(self, response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...) -> None: ...

class AddTransferWarehouseRequest(_message.Message):
    __slots__ = ["delivery_warehouse_id", "warehouse_herarchy_ids", "context"]
    DELIVERY_WAREHOUSE_ID_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_HERARCHY_IDS_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    delivery_warehouse_id: str
    warehouse_herarchy_ids: _containers.RepeatedScalarFieldContainer[str]
    context: _base_pb2.Context
    def __init__(
        self,
        delivery_warehouse_id: _Optional[str] = ...,
        warehouse_herarchy_ids: _Optional[_Iterable[str]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class AddTransferWarehouseResponse(_message.Message):
    __slots__ = ["delivery_warehouse", "response_standard"]
    DELIVERY_WAREHOUSE_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    delivery_warehouse: DeliveryWarehouse
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        delivery_warehouse: _Optional[_Union[DeliveryWarehouse, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...

class RemoveTransferWarehouseRequest(_message.Message):
    __slots__ = ["delivery_warehouse_id", "warehouse_herarchy_ids", "context"]
    DELIVERY_WAREHOUSE_ID_FIELD_NUMBER: _ClassVar[int]
    WAREHOUSE_HERARCHY_IDS_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    delivery_warehouse_id: str
    warehouse_herarchy_ids: _containers.RepeatedScalarFieldContainer[str]
    context: _base_pb2.Context
    def __init__(
        self,
        delivery_warehouse_id: _Optional[str] = ...,
        warehouse_herarchy_ids: _Optional[_Iterable[str]] = ...,
        context: _Optional[_Union[_base_pb2.Context, _Mapping]] = ...,
    ) -> None: ...

class RemoveTransferWarehouseResponse(_message.Message):
    __slots__ = ["delivery_warehouse", "response_standard"]
    DELIVERY_WAREHOUSE_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_STANDARD_FIELD_NUMBER: _ClassVar[int]
    delivery_warehouse: DeliveryWarehouse
    response_standard: _base_pb2.ResponseStandard
    def __init__(
        self,
        delivery_warehouse: _Optional[_Union[DeliveryWarehouse, _Mapping]] = ...,
        response_standard: _Optional[_Union[_base_pb2.ResponseStandard, _Mapping]] = ...,
    ) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/stock/uom.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from omni.pro.protos.common import base_pb2 as common_dot_base__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x12v1/stock/uom.proto\x12\x1dpro.omni.oms.api.v1.stock.uom\x1a\x11\x63ommon/base.proto\x1a\x1egoogle/protobuf/wrappers.proto"\xc3\x01\n\x03Uom\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x12\n\nuom_doc_id\x18\x02 \x01(\t\x12\x0c\n\x04\x63ode\x18\x03 \x01(\t\x12\x0c\n\x04name\x18\x04 \x01(\t\x12*\n\x06\x61\x63tive\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x13\n\x0b\x65xternal_id\x18\x06 \x01(\t\x12?\n\x0cobject_audit\x18\x07 \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\x8f\x01\n\x10UomCreateRequest\x12\x12\n\nuom_doc_id\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x13\n\x0b\x65xternal_id\x18\x04 \x01(\t\x12\x36\n\x07\x63ontext\x18\x05 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x86\x01\n\x11UomCreateResponse\x12@\n\x08response\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12/\n\x03uom\x18\x02 \x01(\x0b\x32".pro.omni.oms.api.v1.stock.uom.Uom"\xec\x02\n\x0eUomReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\x05\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xc9\x01\n\x0fUomReadResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12\x30\n\x04uoms\x18\x03 \x03(\x0b\x32".pro.omni.oms.api.v1.stock.uom.Uom"{\n\x10UomUpdateRequest\x12/\n\x03uom\x18\x01 \x01(\x0b\x32".pro.omni.oms.api.v1.stock.uom.Uom\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x86\x01\n\x11UomUpdateResponse\x12@\n\x08response\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12/\n\x03uom\x18\x02 \x01(\x0b\x32".pro.omni.oms.api.v1.stock.uom.Uom"V\n\x10UomDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"U\n\x11UomDeleteResponse\x12@\n\x08response\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard2\xce\x03\n\nUomService\x12p\n\tUomCreate\x12/.pro.omni.oms.api.v1.stock.uom.UomCreateRequest\x1a\x30.pro.omni.oms.api.v1.stock.uom.UomCreateResponse"\x00\x12j\n\x07UomRead\x12-.pro.omni.oms.api.v1.stock.uom.UomReadRequest\x1a..pro.omni.oms.api.v1.stock.uom.UomReadResponse"\x00\x12p\n\tUomUpdate\x12/.pro.omni.oms.api.v1.stock.uom.UomUpdateRequest\x1a\x30.pro.omni.oms.api.v1.stock.uom.UomUpdateResponse"\x00\x12p\n\tUomDelete\x12/.pro.omni.oms.api.v1.stock.uom.UomDeleteRequest\x1a\x30.pro.omni.oms.api.v1.stock.uom.UomDeleteResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.stock.uom_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_UOM"]._serialized_start = 105
    _globals["_UOM"]._serialized_end = 300
    _globals["_UOMCREATEREQUEST"]._serialized_start = 303
    _globals["_UOMCREATEREQUEST"]._serialized_end = 446
    _globals["_UOMCREATERESPONSE"]._serialized_start = 449
    _globals["_UOMCREATERESPONSE"]._serialized_end = 583
    _globals["_UOMREADREQUEST"]._serialized_start = 586
    _globals["_UOMREADREQUEST"]._serialized_end = 950
    _globals["_UOMREADRESPONSE"]._serialized_start = 953
    _globals["_UOMREADRESPONSE"]._serialized_end = 1154
    _globals["_UOMUPDATEREQUEST"]._serialized_start = 1156
    _globals["_UOMUPDATEREQUEST"]._serialized_end = 1279
    _globals["_UOMUPDATERESPONSE"]._serialized_start = 1282
    _globals["_UOMUPDATERESPONSE"]._serialized_end = 1416
    _globals["_UOMDELETEREQUEST"]._serialized_start = 1418
    _globals["_UOMDELETEREQUEST"]._serialized_end = 1504
    _globals["_UOMDELETERESPONSE"]._serialized_start = 1506
    _globals["_UOMDELETERESPONSE"]._serialized_end = 1591
    _globals["_UOMSERVICE"]._serialized_start = 1594
    _globals["_UOMSERVICE"]._serialized_end = 2056
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/rules/compute_operation.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from omni.pro.protos.common import base_pb2 as common_dot_base__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n v1/rules/compute_operation.proto\x12+pro.omni.oms.api.v1.rules.compute_operation\x1a\x11\x63ommon/base.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1egoogle/protobuf/wrappers.proto"\x9d\x01\n\x11\x43omputeMethodData\x12-\n\x0c\x63\x61rt_details\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\x12&\n\x05items\x18\x02 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x31\n\x10shipping_details\x18\x03 \x01(\x0b\x32\x17.google.protobuf.Struct"\xad\x01\n\x14\x43omputeMethodRequest\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12L\n\x04\x64\x61ta\x18\x02 \x03(\x0b\x32>.pro.omni.oms.api.v1.rules.compute_operation.ComputeMethodData\x12\x36\n\x07\x63ontext\x18\x03 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x8e\x01\n\x15\x43omputeMethodResponse\x12*\n\x06result\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.ListValue\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xae\x01\n"ComputeMethodStockAvailableRequest\x12(\n\x07methods\x18\x01 \x03(\x0b\x32\x17.google.protobuf.Struct\x12&\n\x05items\x18\x02 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x36\n\x07\x63ontext\x18\x03 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x9c\x01\n#ComputeMethodStockAvailableResponse\x12*\n\x06result\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.ListValue\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xe2\x01\n\x1a\x43omputeDeliveryCarrierData\x12(\n\x07\x61\x64\x64ress\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\x12)\n\x08products\x18\x02 \x03(\x0b\x32\x17.google.protobuf.Struct\x12\x18\n\x10warehouse_sql_id\x18\x03 \x01(\x05\x12\x1e\n\x16\x64\x65livery_method_doc_id\x18\x04 \x01(\t\x12\x35\n\x11internal_transfer\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.BoolValue"\xae\x01\n\x1d\x43omputeDeliveryCarrierRequest\x12U\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32G.pro.omni.oms.api.v1.rules.compute_operation.ComputeDeliveryCarrierData\x12\x36\n\x07\x63ontext\x18\x03 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x83\x01\n\x1e\x43omputeDeliveryCarrierResponse\x12\x16\n\x0e\x63\x61rrier_sql_id\x18\x01 \x01(\x05\x12I\n\x11response_standard\x18\x03 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard2\xaf\x04\n\x17\x43omputeOperationService\x12\x98\x01\n\rComputeMethod\x12\x41.pro.omni.oms.api.v1.rules.compute_operation.ComputeMethodRequest\x1a\x42.pro.omni.oms.api.v1.rules.compute_operation.ComputeMethodResponse"\x00\x12\xc2\x01\n\x1b\x43omputeMethodStockAvailable\x12O.pro.omni.oms.api.v1.rules.compute_operation.ComputeMethodStockAvailableRequest\x1aP.pro.omni.oms.api.v1.rules.compute_operation.ComputeMethodStockAvailableResponse"\x00\x12\xb3\x01\n\x16\x43omputeDeliveryCarrier\x12J.pro.omni.oms.api.v1.rules.compute_operation.ComputeDeliveryCarrierRequest\x1aK.pro.omni.oms.api.v1.rules.compute_operation.ComputeDeliveryCarrierResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.rules.compute_operation_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_COMPUTEMETHODDATA"]._serialized_start = 163
    _globals["_COMPUTEMETHODDATA"]._serialized_end = 320
    _globals["_COMPUTEMETHODREQUEST"]._serialized_start = 323
    _globals["_COMPUTEMETHODREQUEST"]._serialized_end = 496
    _globals["_COMPUTEMETHODRESPONSE"]._serialized_start = 499
    _globals["_COMPUTEMETHODRESPONSE"]._serialized_end = 641
    _globals["_COMPUTEMETHODSTOCKAVAILABLEREQUEST"]._serialized_start = 644
    _globals["_COMPUTEMETHODSTOCKAVAILABLEREQUEST"]._serialized_end = 818
    _globals["_COMPUTEMETHODSTOCKAVAILABLERESPONSE"]._serialized_start = 821
    _globals["_COMPUTEMETHODSTOCKAVAILABLERESPONSE"]._serialized_end = 977
    _globals["_COMPUTEDELIVERYCARRIERDATA"]._serialized_start = 980
    _globals["_COMPUTEDELIVERYCARRIERDATA"]._serialized_end = 1206
    _globals["_COMPUTEDELIVERYCARRIERREQUEST"]._serialized_start = 1209
    _globals["_COMPUTEDELIVERYCARRIERREQUEST"]._serialized_end = 1383
    _globals["_COMPUTEDELIVERYCARRIERRESPONSE"]._serialized_start = 1386
    _globals["_COMPUTEDELIVERYCARRIERRESPONSE"]._serialized_end = 1517
    _globals["_COMPUTEOPERATIONSERVICE"]._serialized_start = 1520
    _globals["_COMPUTEOPERATIONSERVICE"]._serialized_end = 2079
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from omni.pro.protos.v1.rules import product_pb2 as v1_dot_rules_dot_product__pb2


class ProductServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ProductCreate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.product.ProductService/ProductCreate",
            request_serializer=v1_dot_rules_dot_product__pb2.ProductCreateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_product__pb2.ProductCreateResponse.FromString,
        )
        self.ProductRead = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.product.ProductService/ProductRead",
            request_serializer=v1_dot_rules_dot_product__pb2.ProductReadRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_product__pb2.ProductReadResponse.FromString,
        )
        self.ProductUpdate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.product.ProductService/ProductUpdate",
            request_serializer=v1_dot_rules_dot_product__pb2.ProductUpdateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_product__pb2.ProductUpdateResponse.FromString,
        )
        self.ProductDelete = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.product.ProductService/ProductDelete",
            request_serializer=v1_dot_rules_dot_product__pb2.ProductDeleteRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_product__pb2.ProductDeleteResponse.FromString,
        )


class ProductServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def ProductCreate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ProductRead(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ProductUpdate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ProductDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_ProductServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "ProductCreate": grpc.unary_unary_rpc_method_handler(
            servicer.ProductCreate,
            request_deserializer=v1_dot_rules_dot_product__pb2.ProductCreateRequest.FromString,
            response_serializer=v1_dot_rules_dot_product__pb2.ProductCreateResponse.SerializeToString,
        ),
        "ProductRead": grpc.unary_unary_rpc_method_handler(
            servicer.ProductRead,
            request_deserializer=v1_dot_rules_dot_product__pb2.ProductReadRequest.FromString,
            response_serializer=v1_dot_rules_dot_product__pb2.ProductReadResponse.SerializeToString,
        ),
        "ProductUpdate": grpc.unary_unary_rpc_method_handler(
            servicer.ProductUpdate,
            request_deserializer=v1_dot_rules_dot_product__pb2.ProductUpdateRequest.FromString,
            response_serializer=v1_dot_rules_dot_product__pb2.ProductUpdateResponse.SerializeToString,
        ),
        "ProductDelete": grpc.unary_unary_rpc_method_handler(
            servicer.ProductDelete,
            request_deserializer=v1_dot_rules_dot_product__pb2.ProductDeleteRequest.FromString,
            response_serializer=v1_dot_rules_dot_product__pb2.ProductDeleteResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "pro.omni.oms.api.v1.rules.product.ProductService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class ProductService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def ProductCreate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.product.ProductService/ProductCreate",
            v1_dot_rules_dot_product__pb2.ProductCreateRequest.SerializeToString,
            v1_dot_rules_dot_product__pb2.ProductCreateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ProductRead(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.product.ProductService/ProductRead",
            v1_dot_rules_dot_product__pb2.ProductReadRequest.SerializeToString,
            v1_dot_rules_dot_product__pb2.ProductReadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ProductUpdate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.product.ProductService/ProductUpdate",
            v1_dot_rules_dot_product__pb2.ProductUpdateRequest.SerializeToString,
            v1_dot_rules_dot_product__pb2.ProductUpdateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ProductDelete(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.product.ProductService/ProductDelete",
            v1_dot_rules_dot_product__pb2.ProductDeleteRequest.SerializeToString,
            v1_dot_rules_dot_product__pb2.ProductDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from omni.pro.protos.v1.utilities import sequence_pb2 as v1_dot_utilities_dot_sequence__pb2


class SequenceServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SequenceCreate = channel.unary_unary(
            "/pro.omni.oms.api.v1.utilities.sequence.SequenceService/SequenceCreate",
            request_serializer=v1_dot_utilities_dot_sequence__pb2.SequenceCreateRequest.SerializeToString,
            response_deserializer=v1_dot_utilities_dot_sequence__pb2.SequenceCreateResponse.FromString,
        )
        self.SequenceRead = channel.unary_unary(
            "/pro.omni.oms.api.v1.utilities.sequence.SequenceService/SequenceRead",
            request_serializer=v1_dot_utilities_dot_sequence__pb2.SequenceReadRequest.SerializeToString,
            response_deserializer=v1_dot_utilities_dot_sequence__pb2.SequenceReadResponse.FromString,
        )
        self.SequenceUpdate = channel.unary_unary(
            "/pro.omni.oms.api.v1.utilities.sequence.SequenceService/SequenceUpdate",
            request_serializer=v1_dot_utilities_dot_sequence__pb2.SequenceUpdateRequest.SerializeToString,
            response_deserializer=v1_dot_utilities_dot_sequence__pb2.SequenceUpdateResponse.FromString,
        )
        self.SequenceDelete = channel.unary_unary(
            "/pro.omni.oms.api.v1.utilities.sequence.SequenceService/SequenceDelete",
            request_serializer=v1_dot_utilities_dot_sequence__pb2.SequenceDeleteRequest.SerializeToString,
            response_deserializer=v1_dot_utilities_dot_sequence__pb2.SequenceDeleteResponse.FromString,
        )
        self.NextById = channel.unary_unary(
            "/pro.omni.oms.api.v1.utilities.sequence.SequenceService/NextById",
            request_serializer=v1_dot_utilities_dot_sequence__pb2.NextByIdRequest.SerializeToString,
            response_deserializer=v1_dot_utilities_dot_sequence__pb2.NextResponse.FromString,
        )
        self.NextByCode = channel.unary_unary(
            "/pro.omni.oms.api.v1.utilities.sequence.SequenceService/NextByCode",
            request_serializer=v1_dot_utilities_dot_sequence__pb2.NextByCodeRequest.SerializeToString,
            response_deserializer=v1_dot_utilities_dot_sequence__pb2.NextResponse.FromString,
        )


class SequenceServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def SequenceCreate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SequenceRead(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SequenceUpdate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SequenceDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def NextById(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def NextByCode(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_SequenceServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "SequenceCreate": grpc.unary_unary_rpc_method_handler(
            servicer.SequenceCreate,
            request_deserializer=v1_dot_utilities_dot_sequence__pb2.SequenceCreateRequest.FromString,
            response_serializer=v1_dot_utilities_dot_sequence__pb2.SequenceCreateResponse.SerializeToString,
        ),
        "SequenceRead": grpc.unary_unary_rpc_method_handler(
            servicer.SequenceRead,
            request_deserializer=v1_dot_utilities_dot_sequence__pb2.SequenceReadRequest.FromString,
            response_serializer=v1_dot_utilities_dot_sequence__pb2.SequenceReadResponse.SerializeToString,
        ),
        "SequenceUpdate": grpc.unary_unary_rpc_method_handler(
            servicer.SequenceUpdate,
            request_deserializer=v1_dot_utilities_dot_sequence__pb2.SequenceUpdateRequest.FromString,
            response_serializer=v1_dot_utilities_dot_sequence__pb2.SequenceUpdateResponse.SerializeToString,
        ),
        "SequenceDelete": grpc.unary_unary_rpc_method_handler(
            servicer.SequenceDelete,
            request_deserializer=v1_dot_utilities_dot_sequence__pb2.SequenceDeleteRequest.FromString,
            response_serializer=v1_dot_utilities_dot_sequence__pb2.SequenceDeleteResponse.SerializeToString,
        ),
        "NextById": grpc.unary_unary_rpc_method_handler(
            servicer.NextById,
            request_deserializer=v1_dot_utilities_dot_sequence__pb2.NextByIdRequest.FromString,
            response_serializer=v1_dot_utilities_dot_sequence__pb2.NextResponse.SerializeToString,
        ),
        "NextByCode": grpc.unary_unary_rpc_method_handler(
            servicer.NextByCode,
            request_deserializer=v1_dot_utilities_dot_sequence__pb2.NextByCodeRequest.FromString,
            response_serializer=v1_dot_utilities_dot_sequence__pb2.NextResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "pro.omni.oms.api.v1.utilities.sequence.SequenceService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class SequenceService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def SequenceCreate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.utilities.sequence.SequenceService/SequenceCreate",
            v1_dot_utilities_dot_sequence__pb2.SequenceCreateRequest.SerializeToString,
            v1_dot_utilities_dot_sequence__pb2.SequenceCreateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def SequenceRead(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.utilities.sequence.SequenceService/SequenceRead",
            v1_dot_utilities_dot_sequence__pb2.SequenceReadRequest.SerializeToString,
            v1_dot_utilities_dot_sequence__pb2.SequenceReadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def SequenceUpdate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.utilities.sequence.SequenceService/SequenceUpdate",
            v1_dot_utilities_dot_sequence__pb2.SequenceUpdateRequest.SerializeToString,
            v1_dot_utilities_dot_sequence__pb2.SequenceUpdateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def SequenceDelete(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.utilities.sequence.SequenceService/SequenceDelete",
            v1_dot_utilities_dot_sequence__pb2.SequenceDeleteRequest.SerializeToString,
            v1_dot_utilities_dot_sequence__pb2.SequenceDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def NextById(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.utilities.sequence.SequenceService/NextById",
            v1_dot_utilities_dot_sequence__pb2.NextByIdRequest.SerializeToString,
            v1_dot_utilities_dot_sequence__pb2.NextResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def NextByCode(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.utilities.sequence.SequenceService/NextByCode",
            v1_dot_utilities_dot_sequence__pb2.NextByCodeRequest.SerializeToString,
            v1_dot_utilities_dot_sequence__pb2.NextResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/rules/special_conditions.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from omni.pro.protos.common import base_pb2 as common_dot_base__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n!v1/rules/special_conditions.proto\x12,pro.omni.oms.api.v1.rules.special_conditions\x1a\x11\x63ommon/base.proto\x1a\x1egoogle/protobuf/wrappers.proto"j\n\x17SpecialConditionsValues\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x15\n\rfamily_doc_id\x18\x02 \x01(\t\x12\x12\n\ngroup_code\x18\x03 \x01(\t\x12\x16\n\x0e\x61ttribute_code\x18\x04 \x01(\t"\x92\x02\n\x11SpecialConditions\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x61\n\x12special_conditions\x18\x03 \x03(\x0b\x32\x45.pro.omni.oms.api.v1.rules.special_conditions.SpecialConditionsValues\x12*\n\x06\x61\x63tive\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x13\n\x0b\x65xternal_id\x18\x05 \x01(\t\x12?\n\x0cobject_audit\x18\x06 \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\xde\x01\n\x1eSpecialConditionsCreateRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x61\n\x12special_conditions\x18\x02 \x03(\x0b\x32\x45.pro.omni.oms.api.v1.rules.special_conditions.SpecialConditionsValues\x12\x13\n\x0b\x65xternal_id\x18\x03 \x01(\t\x12\x36\n\x07\x63ontext\x18\x04 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xc9\x01\n\x1fSpecialConditionsCreateResponse\x12[\n\x12special_conditions\x18\x01 \x01(\x0b\x32?.pro.omni.oms.api.v1.rules.special_conditions.SpecialConditions\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xfa\x02\n\x1cSpecialConditionsReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x83\x02\n\x1dSpecialConditionsReadResponse\x12\\\n\x13\x64\x65livery_categories\x18\x01 \x03(\x0b\x32?.pro.omni.oms.api.v1.rules.special_conditions.SpecialConditions\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12I\n\x11response_standard\x18\x03 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xb4\x01\n\x1eSpecialConditionsUpdateRequest\x12Z\n\x11\x64\x65livery_category\x18\x01 \x01(\x0b\x32?.pro.omni.oms.api.v1.rules.special_conditions.SpecialConditions\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xc8\x01\n\x1fSpecialConditionsUpdateResponse\x12Z\n\x11\x64\x65livery_category\x18\x01 \x01(\x0b\x32?.pro.omni.oms.api.v1.rules.special_conditions.SpecialConditions\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"d\n\x1eSpecialConditionsDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"l\n\x1fSpecialConditionsDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard2\x80\x06\n\x18SpecialConditionsService\x12\xb8\x01\n\x17SpecialConditionsCreate\x12L.pro.omni.oms.api.v1.rules.special_conditions.SpecialConditionsCreateRequest\x1aM.pro.omni.oms.api.v1.rules.special_conditions.SpecialConditionsCreateResponse"\x00\x12\xb2\x01\n\x15SpecialConditionsRead\x12J.pro.omni.oms.api.v1.rules.special_conditions.SpecialConditionsReadRequest\x1aK.pro.omni.oms.api.v1.rules.special_conditions.SpecialConditionsReadResponse"\x00\x12\xb8\x01\n\x17SpecialConditionsUpdate\x12L.pro.omni.oms.api.v1.rules.special_conditions.SpecialConditionsUpdateRequest\x1aM.pro.omni.oms.api.v1.rules.special_conditions.SpecialConditionsUpdateResponse"\x00\x12\xb8\x01\n\x17SpecialConditionsDelete\x12L.pro.omni.oms.api.v1.rules.special_conditions.SpecialConditionsDeleteRequest\x1aM.pro.omni.oms.api.v1.rules.special_conditions.SpecialConditionsDeleteResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.rules.special_conditions_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_SPECIALCONDITIONSVALUES"]._serialized_start = 134
    _globals["_SPECIALCONDITIONSVALUES"]._serialized_end = 240
    _globals["_SPECIALCONDITIONS"]._serialized_start = 243
    _globals["_SPECIALCONDITIONS"]._serialized_end = 517
    _globals["_SPECIALCONDITIONSCREATEREQUEST"]._serialized_start = 520
    _globals["_SPECIALCONDITIONSCREATEREQUEST"]._serialized_end = 742
    _globals["_SPECIALCONDITIONSCREATERESPONSE"]._serialized_start = 745
    _globals["_SPECIALCONDITIONSCREATERESPONSE"]._serialized_end = 946
    _globals["_SPECIALCONDITIONSREADREQUEST"]._serialized_start = 949
    _globals["_SPECIALCONDITIONSREADREQUEST"]._serialized_end = 1327
    _globals["_SPECIALCONDITIONSREADRESPONSE"]._serialized_start = 1330
    _globals["_SPECIALCONDITIONSREADRESPONSE"]._serialized_end = 1589
    _globals["_SPECIALCONDITIONSUPDATEREQUEST"]._serialized_start = 1592
    _globals["_SPECIALCONDITIONSUPDATEREQUEST"]._serialized_end = 1772
    _globals["_SPECIALCONDITIONSUPDATERESPONSE"]._serialized_start = 1775
    _globals["_SPECIALCONDITIONSUPDATERESPONSE"]._serialized_end = 1975
    _globals["_SPECIALCONDITIONSDELETEREQUEST"]._serialized_start = 1977
    _globals["_SPECIALCONDITIONSDELETEREQUEST"]._serialized_end = 2077
    _globals["_SPECIALCONDITIONSDELETERESPONSE"]._serialized_start = 2079
    _globals["_SPECIALCONDITIONSDELETERESPONSE"]._serialized_end = 2187
    _globals["_SPECIALCONDITIONSSERVICE"]._serialized_start = 2190
    _globals["_SPECIALCONDITIONSSERVICE"]._serialized_end = 2958
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/rules/warehouse.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from omni.pro.protos.common import base_pb2 as common_dot_base__pb2
from omni.pro.protos.v1.rules import delivery_schedule_pb2 as v1_dot_rules_dot_delivery__schedule__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x18v1/rules/warehouse.proto\x12#pro.omni.oms.api.v1.rules.warehouse\x1a\x11\x63ommon/base.proto\x1a v1/rules/delivery_schedule.proto"\xb3\x02\n\tWarehouse\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0c\n\x04\x63ode\x18\x03 \x01(\t\x12\x18\n\x10warehouse_sql_id\x18\x04 \x01(\x05\x12\x1a\n\x12locality_available\x18\x05 \x01(\t\x12\x18\n\x10loc_stock_sql_id\x18\x06 \x01(\x05\x12\x13\n\x0b\x65xternal_id\x18\x07 \x01(\t\x12X\n\x11schedule_template\x18\x08 \x01(\x0b\x32=.pro.omni.oms.api.v1.rules.delivery_schedule.DeliverySchedule\x12?\n\x0cobject_audit\x18\t \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\xef\x01\n\x16WarehouseCreateRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12\x18\n\x10warehouse_sql_id\x18\x03 \x01(\x05\x12\x1a\n\x12locality_available\x18\x04 \x01(\t\x12\x18\n\x10loc_stock_sql_id\x18\x05 \x01(\x05\x12\x13\n\x0b\x65xternal_id\x18\x06 \x01(\t\x12\x1c\n\x14schedule_template_id\x18\x07 \x01(\t\x12\x36\n\x07\x63ontext\x18\x08 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xa7\x01\n\x17WarehouseCreateResponse\x12\x41\n\twarehouse\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.v1.rules.warehouse.Warehouse\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\xf2\x02\n\x14WarehouseReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\t\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xe1\x01\n\x15WarehouseReadResponse\x12\x42\n\nwarehouses\x18\x01 \x03(\x0b\x32..pro.omni.oms.api.v1.rules.warehouse.Warehouse\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12I\n\x11response_standard\x18\x03 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\x93\x01\n\x16WarehouseUpdateRequest\x12\x41\n\twarehouse\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.v1.rules.warehouse.Warehouse\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xa7\x01\n\x17WarehouseUpdateResponse\x12\x41\n\twarehouse\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.v1.rules.warehouse.Warehouse\x12I\n\x11response_standard\x18\x02 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"v\n\x16WarehouseDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x18\n\x10warehouse_sql_id\x18\x02 \x01(\x05\x12\x36\n\x07\x63ontext\x18\x03 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"d\n\x17WarehouseDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard2\xd0\x04\n\x10WarehouseService\x12\x8e\x01\n\x0fWarehouseCreate\x12;.pro.omni.oms.api.v1.rules.warehouse.WarehouseCreateRequest\x1a<.pro.omni.oms.api.v1.rules.warehouse.WarehouseCreateResponse"\x00\x12\x88\x01\n\rWarehouseRead\x12\x39.pro.omni.oms.api.v1.rules.warehouse.WarehouseReadRequest\x1a:.pro.omni.oms.api.v1.rules.warehouse.WarehouseReadResponse"\x00\x12\x8e\x01\n\x0fWarehouseUpdate\x12;.pro.omni.oms.api.v1.rules.warehouse.WarehouseUpdateRequest\x1a<.pro.omni.oms.api.v1.rules.warehouse.WarehouseUpdateResponse"\x00\x12\x8e\x01\n\x0fWarehouseDelete\x12;.pro.omni.oms.api.v1.rules.warehouse.WarehouseDeleteRequest\x1a<.pro.omni.oms.api.v1.rules.warehouse.WarehouseDeleteResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.rules.warehouse_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_WAREHOUSE"]._serialized_start = 119
    _globals["_WAREHOUSE"]._serialized_end = 426
    _globals["_WAREHOUSECREATEREQUEST"]._serialized_start = 429
    _globals["_WAREHOUSECREATEREQUEST"]._serialized_end = 668
    _globals["_WAREHOUSECREATERESPONSE"]._serialized_start = 671
    _globals["_WAREHOUSECREATERESPONSE"]._serialized_end = 838
    _globals["_WAREHOUSEREADREQUEST"]._serialized_start = 841
    _globals["_WAREHOUSEREADREQUEST"]._serialized_end = 1211
    _globals["_WAREHOUSEREADRESPONSE"]._serialized_start = 1214
    _globals["_WAREHOUSEREADRESPONSE"]._serialized_end = 1439
    _globals["_WAREHOUSEUPDATEREQUEST"]._serialized_start = 1442
    _globals["_WAREHOUSEUPDATEREQUEST"]._serialized_end = 1589
    _globals["_WAREHOUSEUPDATERESPONSE"]._serialized_start = 1592
    _globals["_WAREHOUSEUPDATERESPONSE"]._serialized_end = 1759
    _globals["_WAREHOUSEDELETEREQUEST"]._serialized_start = 1761
    _globals["_WAREHOUSEDELETEREQUEST"]._serialized_end = 1879
    _globals["_WAREHOUSEDELETERESPONSE"]._serialized_start = 1881
    _globals["_WAREHOUSEDELETERESPONSE"]._serialized_end = 1981
    _globals["_WAREHOUSESERVICE"]._serialized_start = 1984
    _globals["_WAREHOUSESERVICE"]._serialized_end = 2576
# @@protoc_insertion_point(module_scope)



import hashlib
import json
import logging
import math
import secrets
import string
import time
import unicodedata
from datetime import datetime
from functools import reduce, wraps

from bson import ObjectId
from google.protobuf.timestamp_pb2 import Timestamp
from omni.pro.exceptions import AlreadyExistError, NotFoundError
from omni.pro.stack import ExitStackDocument

logger = logging.getLogger(__name__)

DEFAULT_RECORD_LIMIT = 10000


def generate_strong_password(length=12):
    if length < 8:
        raise ValueError("The password length must be at least 8 characters.")

    punctuation = "!#$%&()*?@[]}{"
    characters = string.ascii_letters + string.digits + punctuation

    password = [
        secrets.choice(string.ascii_lowercase),
        secrets.choice(string.ascii_uppercase),
        secrets.choice(string.digits),
        secrets.choice(punctuation),
    ]

    for _ in range(length - 4):
        password.append(secrets.choice(characters))

    secrets.SystemRandom().shuffle(password)
    password_str = "".join(password)

    return password_str


def nested(data: dict, keys: str, default=None):
    """
    Receives a dictionary and a list of keys, and returns the value associated with the keys in order,
    searching the dictionary to any depth, not including lists. in order, searching the dictionary to
    any depth, not including lists. If the key is not found, it returns the default value.
    """
    return reduce(lambda d, key: d.get(key, default) if isinstance(d, dict) else default, keys.split("."), data)


def deep_search(lst, key=None, value=None, default=None):
    """
    Searches an item in a list in depth by key or value.

    Args:
        lst (list): the list to search.
        key (str): The key to search for.
        value: The value to search for.

    Returns:
        The element found or None if it was not found.
    """
    if isinstance(lst, dict):
        if key in lst and lst[key] == value:
            return lst
        for v in lst.values():
            if isinstance(v, (dict, list)):
                result = deep_search(v, key=key, value=value)
                if result is not None:
                    return result
    elif isinstance(lst, list):
        for elem in lst:
            if isinstance(elem, (dict, list)):
                result = deep_search(elem, key=key, value=value)
                if result is not None:
                    return result
            elif key is None and elem == value:
                return elem
            elif isinstance(elem, dict) and key in elem and elem[key] == value:
                return elem
    return default


def load_file_module(file_path: str, class_name: str):
    """
    Loads a module from a file.

    Args:
        file_path (str): The path to the file.
        class_name (str): The name of the class.

    Returns:
        The module loaded.
    """
    import importlib.util

    spec = importlib.util.spec_from_file_location(class_name, file_path)
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    return module


def paginate_list(objects_list, page_num, per_page, filters=None):
    filtered_list = objects_list
    if filters:
        filtered_list = [o for o in objects_list if o.attr == filters["attr"]]
    start = (page_num - 1) * per_page
    end = start + per_page
    paginated_list = filtered_list[start:end]
    total_pages = int(math.ceil(len(filtered_list) / per_page))
    return paginated_list, total_pages


def parse_bool(value):
    if isinstance(value, bool):
        return value
    if isinstance(value, str):
        return value.lower() in ("yes", "true", "t", "1", "y")
    return bool(value)


def to_camel_case(text: str):
    # Divide el texto por guiones bajos y une cada palabra con la primera letra en mayúsculas
    return "".join(word.capitalize() for word in text.split("_"))


def normalize(value):
    return unicodedata.normalize("NFKD", str(value).strip()).encode("ascii", "ignore").decode("UTF-8")


class HTTPStatus(object):
    OK = 200
    CREATED = 201
    ACCEPTED = 202
    NO_CONTENT = 204
    BAD_REQUEST = 400
    UNAUTHORIZED = 401
    FORBIDDEN = 403
    NOT_FOUND = 404
    METHOD_NOT_ALLOWED = 405
    CONFLICT = 409
    UNPROCESSABLE_ENTITY = 422
    INTERNAL_SERVER_ERROR = 500


class Resource(object):
    AWS_COGNITO = "cognito"
    MONGODB = "mongodb"
    POSTGRES = "postgres"
    AWS_S3 = "s3"


def generate_hash(obj):
    # Convertir el objeto a una cadena con claves ordenadas
    obj_str = json.dumps(obj, sort_keys=True)

    # Generar un hash SHA-256
    return hashlib.sha256(obj_str.encode()).hexdigest()


def merge_and_remove_key(data_dict, key_to_remove):
    """
    Merge and remove key from a dictionary.
    Fusiona y elimina una clave de un diccionario.

    Args:
    data_dict (dict): The original dictionary. El diccionario original.
    key_to_remove (str): The key to be removed and merged. La clave a eliminar y fusionar.

    Returns:
    dict: A dictionary with the key removed and its values merged. Un diccionario con la clave eliminada y sus valores fusionados.
    """
    extracted_data = data_dict.pop(key_to_remove, {})
    return {**extracted_data, **data_dict}


def measure_time(function):
    """
    Decorator to measure the execution time of a function.
    Decorador para medir el tiempo de ejecución de una función.

    Args:
    function (callable): The function to be measured.
    función (callable): La función a medir.

    Returns:
    callable: The wrapped function that measures its execution time.
    callable: La función envuelta que mide su tiempo de ejecución.
    """

    @wraps(function)
    def measured_function(*args, **kwargs):
        """
        The wrapped function that calculates the time taken to execute the original function.
        La función envuelta que calcula el tiempo que tarda en ejecutarse la función original.

        Args:
        *args: Variable length argument list for the original function.
        *args: Lista de argumentos de longitud variable para la función original.

        **kwargs: Arbitrary keyword arguments for the original function.
        **kwargs: Argumentos de palabras clave arbitrarias para la función original.

        Returns:
        The result of the original function.
        El resultado de la función original.
        """
        start = time.time()
        result = function(*args, **kwargs)
        elapsed_time = time.time() - start
        logger.info(f"Func: {function.__qualname__} - Time: {elapsed_time:.2f} seconds")
        return result

    return measured_function


def add_or_remove_document_relations(
    context,
    document,
    exsitent_relations_list,
    new_relations_list,
    attribute_search,
    request_context,
    element_name,
    element_relation_name,
    multiple_params=False,
    params_multiple: tuple = None,
):
    """
    The add_or_remove_document_relations function process and get registers to remove and add, to apply changes and return a list of result.
    La función add_or_remove_document_relations procesa y obtiene registros para eliminar y agregar, para aplicar cambios y devolver una lista de resultados.

    Args:
    context: Context of the request. Contexto de la petición.
    document: Class document to validate. Clase documento a validar.
    tenant: Tenant of request. Tenant de la petición.
    exsitent_relations_list: List of realtion to validate. Lista de relaciones a validar.
    new_relations_list: List to add or remove. Lista a agregar o eliminar.
    attribute_search: Attribute to search. Atributo a buscar.
    request_context: Context of the request. Contexto de la petición.
    element_name: Model name. Nombre del modelo.
    element_relation_name: Relation name. Nombre de la relación.
    multiple_params: If the search is by multiple params. Si la búsqueda es por múltiples parámetros.

    Returns:
    The list of relations process.
    La lista de relaciones procesadas.
    """

    relations_list = set([x.__getattribute__(attribute_search) for x in exsitent_relations_list])
    set_new_relations_list = set(
        new_relations_list if multiple_params is False else [x["code"] for x in new_relations_list]
    )

    if multiple_params:
        add_relations_list = [item for item in new_relations_list if item["code"] not in relations_list]
        remove_relations_list = [item for item in exsitent_relations_list if item.code not in set_new_relations_list]
        remove_relations_list = [{key: getattr(item, key) for key in params_multiple} for item in remove_relations_list]
    else:
        add_relations_list = list(set_new_relations_list - relations_list)
        remove_relations_list = list(relations_list - set_new_relations_list)
    result_list = []

    result_list = remove_document_relations(
        context,
        document,
        remove_relations_list,
        exsitent_relations_list,
        attribute_search,
        request_context,
        element_name,
        element_relation_name,
        multiple_params,
    )
    result_list = add_document_relations(
        context,
        document,
        add_relations_list,
        result_list,
        attribute_search,
        request_context,
        element_name,
        element_relation_name,
        multiple_params,
    )
    return result_list


def remove_document_relations(
    context,
    document,
    list_elements,
    list_registers,
    attribute_search,
    request_context,
    element_name,
    element_relation_name,
    multiple_params=False,
):
    """
    The remove_document_relations function remove resgisters of list_registers the elements defined on list_elements.
    La función remove_document_relations elimina registros de list_registers los elementos definidos en list_elements.

    Args:
    context: Context of the request. Contexto de la petición.
    document: Class document to validate. Clase documento a validar.
    list_elements: List of realtion to remove. Lista de relaciones a eliminar.
    list_registers: List to add or remove. Lista a agregar o eliminar.
    attribute_search: Attribute to search. Atributo a buscar.
    request_context: Context of the request. Contexto de la petición.
    element_name: Model name. Nombre del modelo.
    element_relation_name: Relation name. Nombre de la relación.
    multiple_params: If the search is by multiple params. Si la búsqueda es por múltiples parámetros.

    Returns:
    The list of relations process.
    La lista de relaciones procesadas.
    """
    with ExitStackDocument(document_classes=document.reference_list(), db_alias=context.db_name):
        for element in list_elements:
            register = get_register(attribute_search, context, document, element, request_context, multiple_params)
            if register not in list_registers:
                raise NotFoundError(message=f"{element_name} {element} not defined in {element_relation_name}")
            list_registers.remove(register)

        return list_registers


def add_document_relations(
    context,
    document,
    list_elements,
    list_registers,
    attribute_search,
    request_context,
    element_name,
    element_relation_name,
    multiple_params=False,
):
    """
    The add_document_relations function add resgisters to list_registers from elements defined on list_elements.
    La función add_document_relations agrega registros a list_registers de elementos definidos en list_elements.

    Args:
    context: Context of the request. Contexto de la petición.
    document: Class document to validate. Clase documento a validar.
    list_elements: List of realtion to add. Lista de relaciones a agregar.
    list_registers: List to add or remove. Lista a agregar o eliminar.
    attribute_search: Attribute to search. Atributo a buscar.
    request_context: Context of the request. Contexto de la petición.
    element_name: Model name. Nombre del modelo.
    element_relation_name: Relation name. Nombre de la relación.
    multiple_params: If the search is by multiple params. Si la búsqueda es por múltiples parámetros.

    Returns:
    The list of relations process.
    La lista de relaciones procesadas.
    """
    with ExitStackDocument(document_classes=document.reference_list(), db_alias=context.db_name):
        for element in list_elements:
            register = get_register(attribute_search, context, document, element, request_context, multiple_params)
            if not register:
                raise NotFoundError(message=f"{element_name} {element} not found")
            if register in list_registers:
                raise AlreadyExistError(message=f"{element_name} {element} already added in {element_relation_name}")
            list_registers.append(register)

        return list_registers


def get_register(attribute_search, context, document, element, request_context, multiple_params):
    """
    The get_register function get resgister from diferent type search by id or get_or_sync.
    La función get_register obtiene el registro de la búsqueda de diferentes tipos por id o get_or_sync.

    Args:
    attribute_search: Attribute to search. Atributo a buscar.
    context: Context of the request. Contexto de la petición.
    document: Class document to validate. Clase documento a validar.
    element: Element to search. Elemento a buscar.
    request_context: Context of the request. Contexto de la petición.
    multiple_params: If the search is by multiple params. Si la búsqueda es por múltiples parámetros.

    Returns:
    Object of the register.
    Objeto del registro.
    """
    if attribute_search == "id":
        return context.db_manager.get_document(
            context.db_name, request_context.get("tenant"), document, **{attribute_search: element}
        )
    if multiple_params:
        return document.get_or_sync(request_context, **element)
    return document.get_or_sync(request_context, **{attribute_search: element})


def objects_to_integer(data, fields):
    """
    Transform dictionary keys and values according to specified fields.

    Args:
    data (dict): The input dictionary to transform. El diccionario de entrada para transformar.
    fields (set): A set of field names to consider for transformation. Una conjunto de nombres de campo a considerar para la transformación.

    Returns:
    dict: A transformed dictionary with modified keys and values. Un diccionario transformado con claves y valores modificados.
    """
    transformed_data = {}
    for key, value in data.items():
        new_key = key
        if key in fields:
            new_key = key + "_id" if not key.endswith("_id") else key
            if isinstance(value, dict):
                # If any key in the inner dictionary ends with "_doc_id", use its value
                doc_id_key = next((k for k in value.keys() if k.endswith("_doc_id")), None)
                if doc_id_key:
                    transformed_data[new_key] = value[doc_id_key]
                elif "id" in value:
                    transformed_data[new_key] = value["id"]
            else:
                transformed_data[new_key] = value
        else:
            transformed_data[new_key] = value
    return transformed_data


def convert_to_serializable(value):
    """
    Converts various data types to formats suitable for serialization.

    This function aims to handle a variety of data types for serialization purposes,
    such as converting ObjectId to string and datetime to Timestamp. It's designed to be
    extendable for additional data types.

    Parameters
    ----------
    value : any
        The data to be converted. Can be of any type.

    Returns
    -------
    any
        The data in a format suitable for serialization. The specific return type depends
        on the input type. For example, ObjectId is converted to string, datetime to Timestamp.

    Notes
    -----
    - The function currently handles dicts, lists, ObjectId, and datetime types.
    - More data types can be added as elif blocks within the function.
    - For unrecognized types, the original value is returned.

    Examples
    --------
    >>> convert_to_serializable({"id": ObjectId("507f1f77bcf86cd799439011"), "date": datetime(2020, 1, 1)})
    {'id': '507f1f77bcf86cd799439011', 'date': Timestamp(seconds=1577836800, nanos=0)}
    >>> convert_to_serializable([ObjectId("507f1f77bcf86cd799439011"), datetime(2020, 1, 1)])
    ['507f1f77bcf86cd799439011', Timestamp(seconds=1577836800, nanos=0)]
    """
    if isinstance(value, dict):
        return {k: convert_to_serializable(v) for k, v in value.items()}
    elif isinstance(value, list):
        return [convert_to_serializable(v) for v in value]
    elif isinstance(value, ObjectId):
        return str(value)
    elif isinstance(value, datetime):
        return Timestamp().FromDatetime(value)
    # Add more elif statements here for other specific data types
    # Example:
    # elif isinstance(value, CustomType):
    #     return custom_conversion_function(value)
    return value



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
from omni.pro.protos.v1.rules import stock_security_pb2 as v1_dot_rules_dot_stock__security__pb2


class StockSecurityServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.StockSecurityCreate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.stock_security.StockSecurityService/StockSecurityCreate",
            request_serializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityCreateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityCreateResponse.FromString,
        )
        self.StockSecurityRead = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.stock_security.StockSecurityService/StockSecurityRead",
            request_serializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityReadRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityReadResponse.FromString,
        )
        self.StockSecurityUpdate = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.stock_security.StockSecurityService/StockSecurityUpdate",
            request_serializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityUpdateRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityUpdateResponse.FromString,
        )
        self.StockSecurityDelete = channel.unary_unary(
            "/pro.omni.oms.api.v1.rules.stock_security.StockSecurityService/StockSecurityDelete",
            request_serializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityDeleteRequest.SerializeToString,
            response_deserializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityDeleteResponse.FromString,
        )


class StockSecurityServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def StockSecurityCreate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def StockSecurityRead(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def StockSecurityUpdate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def StockSecurityDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_StockSecurityServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "StockSecurityCreate": grpc.unary_unary_rpc_method_handler(
            servicer.StockSecurityCreate,
            request_deserializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityCreateRequest.FromString,
            response_serializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityCreateResponse.SerializeToString,
        ),
        "StockSecurityRead": grpc.unary_unary_rpc_method_handler(
            servicer.StockSecurityRead,
            request_deserializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityReadRequest.FromString,
            response_serializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityReadResponse.SerializeToString,
        ),
        "StockSecurityUpdate": grpc.unary_unary_rpc_method_handler(
            servicer.StockSecurityUpdate,
            request_deserializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityUpdateRequest.FromString,
            response_serializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityUpdateResponse.SerializeToString,
        ),
        "StockSecurityDelete": grpc.unary_unary_rpc_method_handler(
            servicer.StockSecurityDelete,
            request_deserializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityDeleteRequest.FromString,
            response_serializer=v1_dot_rules_dot_stock__security__pb2.StockSecurityDeleteResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "pro.omni.oms.api.v1.rules.stock_security.StockSecurityService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class StockSecurityService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def StockSecurityCreate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.stock_security.StockSecurityService/StockSecurityCreate",
            v1_dot_rules_dot_stock__security__pb2.StockSecurityCreateRequest.SerializeToString,
            v1_dot_rules_dot_stock__security__pb2.StockSecurityCreateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def StockSecurityRead(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.stock_security.StockSecurityService/StockSecurityRead",
            v1_dot_rules_dot_stock__security__pb2.StockSecurityReadRequest.SerializeToString,
            v1_dot_rules_dot_stock__security__pb2.StockSecurityReadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def StockSecurityUpdate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.stock_security.StockSecurityService/StockSecurityUpdate",
            v1_dot_rules_dot_stock__security__pb2.StockSecurityUpdateRequest.SerializeToString,
            v1_dot_rules_dot_stock__security__pb2.StockSecurityUpdateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def StockSecurityDelete(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/pro.omni.oms.api.v1.rules.stock_security.StockSecurityService/StockSecurityDelete",
            v1_dot_rules_dot_stock__security__pb2.StockSecurityDeleteRequest.SerializeToString,
            v1_dot_rules_dot_stock__security__pb2.StockSecurityDeleteResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )
